import React, { useState, useEffect, useRef } from 'react';
import { getAvailability, setAvailability } from './api';

const NameForm = ({ onNameSet }) => {
  const [name, setName] = useState("");

  return (
    <div>
      <form onSubmit={() => onNameSet(name.trim())}>
        <input 
          type="text" 
          onChange={({ target }) => setName(target.value)}
          value={name}
        />
        <input 
          disabled={name === ""} 
          type="submit" 
          value="Submit"
        />
      </form>
    </div>
  )
}

const colors = {
  yes: "#0f0",
  no: "#f00",
  maybe: "#ff0",
}

const nextState = {
  yes: "no",
  no: "maybe",
  maybe: "unknown",
  unknown: "yes",
}

const pad = num => {
  let snum = "" + num
  if (snum.length === 1) snum = "0" + snum
  return snum
}

const Tile = ({ user, month, day, availability, onClick }) => (
  <div 
    onClick={onClick}
    style={{
      display: "flex",
      flexFlow: "column nowrap",
      width: "10em",
      height: "10em",
      border: "1px solid black",
      margin: "2px 2px 2px 2px",
      backgroundColor: colors[availability.filter(({ name }) => name === user)[0]?.status] || "#fff",
    }}
  >
    <span>{pad(month)}/{pad(day)}</span>
    {
      availability
        .filter(({ status }) => status && status !== "unknown")
        .filter(({ name }) => name !== user)
        .map(({ name, status }) => (
          <span 
            key={name} 
            style={{ 
              backgroundColor: colors[status],
              overflow: "hidden",
              textOverflow: "ellipsis",
            }}
          >
            {name}
          </span>
        ))
      }
  </div>
);


const chunk = (arr, len) => {
  const chunks = [];
  for (let i = 0; i < arr.length;) {
    chunks.push(arr.slice(i, i += len));
  }
  return chunks;
}

function App() {
  const [name, setName] = useState(null);
  const [calendar, setCalendar] = useState(null);
  const lastUpdatedRef = useRef(null);

  const updateCalendar = async () => {
    const { availability, lastUpdated } = await getAvailability();
    if (lastUpdated !== lastUpdatedRef.current) {
      // only actually update if there are changes
      lastUpdatedRef.current = lastUpdated;
      setCalendar(availability);
    }
  };

  const advance = async (dates) => {
    const updates = dates.map(({ month, day }) => {
      const currentStatus = calendar
        .find(item => item.month === month && item.day === day)
        ?.availability
        ?.find(item => item.name === name)
        ?.status 
        ?? "unknown"
      const status = nextState[currentStatus] || "unknown";
      return { month, day, status }
    });
    const { lastUpdated, availability } = await setAvailability(name, updates);
    lastUpdatedRef.current = lastUpdated;
    setCalendar(availability);
  };
  
  useEffect(() => { 
    const interval = setInterval(updateCalendar, 5000); // update every few seconds

    updateCalendar(); // do an update to get started

    return () => clearInterval(interval); // cleanup if we unmount
  }, []);


  if (name === null) {
    return <NameForm onNameSet={setName} />
  }

  if (calendar === null) {
    return <span>Loading...</span>
  }

  return (
    <div
      style={{
        display: "flex",
        flexFlow: "column nowrap",
      }}
    >
      <span>Logged in as: {name}</span>
      {
        chunk(calendar, 7).map((row, ind) => (
          <div
            key={ind} 
            style={{
              display: "flex",
              flexFlow: "row nowrap",
              alignItems: "center",
            }}
          >
            <div
              onClick={() => advance(row)}
            >
              WEEK
            </div>
            <div 
              style={{
                display: "flex",
                flexFlow: "row nowrap",
              }}
            >
              {
                row.map(item => (
                  <Tile
                    onClick={() => advance([item])}
                    key={item.day} 
                    user={name} 
                    {...item}
                  />
                ))
              }
            </div>
          </div>
        ))
      }
    </div>
  );
}

export default App;
